const { dao, proposals } = require('../util/contracts');
const read = require('../util/forum/read');

const {
  START_PROPOSAL_ID,
  STOP_PROPOSAL_ID,
  ENABLE_STAKING,
} = process.env;

let enableStaking = true;

// Subscribe to proposal events
const start = async () => {
  if (ENABLE_STAKING === 'false') {
    enableStaking = false;
  } else {
    if (STOP_PROPOSAL_ID) {
      enableStaking = false;
      // Check for status
      const proposal = await proposals.proposals(STOP_PROPOSAL_ID);
      if (proposal.stage === BigInt(5)) {
      // Proposal is accepted
        enableStaking = false;
        console.log(`STOP_PROPOSAL_ID ${STOP_PROPOSAL_ID} proposal accepted. Not starting.`);
      } else if (proposal.stage === BigInt(4)) {
      // Proposal is failed
        console.log(`STOP_PROPOSAL_ID ${STOP_PROPOSAL_ID} proposal is failed. No effect.`);
      } else {
      // Register a ProposalAccepted event handler.
        console.log(`STOP_PROPOSAL_ID ${STOP_PROPOSAL_ID} proposal is stage ${proposal.stage.toString()}. Registering listener.`);
        const proposalAcceptedHandler = (event) => {
          if (event.returnValues.proposalIndex === STOP_PROPOSAL_ID) {
            console.log(`STOP_PROPOSAL_ID ${STOP_PROPOSAL_ID} proposal accepted. Stopping.`);
            enableStaking = false;
            proposals.off('ProposalAccepted', proposalAcceptedHandler);
          }
        };
        proposals.on('ProposalAccepted', proposalAcceptedHandler);
      }
    }
    if (START_PROPOSAL_ID) {
      enableStaking = false;
      // Check for status
      const proposal = await proposals.proposals(START_PROPOSAL_ID);
      if (proposal.stage === BigInt(5)) {
      // Proposal is accepted
        enableStaking = true;
      } else if (proposal.stage === BigInt(4)) {
      // Proposal is failed
        console.log(`START_PROPOSAL_ID ${START_PROPOSAL_ID} proposal is failed. Not starting.`);
      } else {
      // Register a ProposalAccepted event handler.
        console.log(`START_PROPOSAL_ID ${START_PROPOSAL_ID} proposal is stage ${proposal.stage.toString()}. Registering listener.`);
        const proposalAcceptedHandler = (event) => {
          if (event.returnValues.proposalIndex === START_PROPOSAL_ID) {
            console.log(`START_PROPOSAL_ID ${START_PROPOSAL_ID} proposal accepted. Starting.`);
            enableStaking = true;
            proposals.off('ProposalAccepted', proposalAcceptedHandler);
          }
        };
        proposals.on('ProposalAccepted', proposalAcceptedHandler);
      }
    }
  }

  console.log('registering validation pool listener');
  dao.on('ValidationPoolInitiated', async (poolIndex) => {
    console.log('Validation Pool Initiated, index', poolIndex);

    const pool = await dao.validationPools(poolIndex);
    console.log('postId:', pool.postId);

    // Read post from database
    let post;
    try {
      post = await read(pool.postId);
    } catch (e) {
      // Post not found
      console.error(`error: post for validation pool ${poolIndex} not found`);
      return;
    }

    console.log('post.content:', post.content);

    // We have the opportunity to stake for/against this validation pool.
    // To implement the legislative process of upgrading this protocol,
    // the execution of this code can be protected by a given proposal.
    // The code will only execute if the proposal has been accepted.
    if (!enableStaking) return;

    console.log('considering staking');
  });
};

module.exports = {
  start,
};
